
from .custom_exception import CustomException


class NotFoundException(CustomException):

    status_code = 400
    message = "Resource Not Found"

    def __init__(self):
        super().__init__("Resource Not Found")

    def serialize_errors(self):
        return [{"message": self.message}]



from .custom_exception import CustomException


class RequestValidationException(CustomException):

    status_code = 400

    def __init__(self, errors):
        super().__init__('Validation Error')
        self.errors = errors.__dict__["messages"]

    def serialize_errors(self):

        serializedErrors = []
        for field, messages in self.errors.items():

            if isinstance(messages, dict):
                for k, v in messages.items():
                    serializedErrors.append({"message": v[0], "field": field})

            if isinstance(messages, list):
                for msg in messages:
                    serializedErrors.append({"message": msg, "field": field})
        return serializedErrors



from .custom_exception import CustomException
from .request_validation_exception import RequestValidationException
from .bad_request_exception import BadRequestException
from .database_exception import DatabaseException
from .unauthorized_exception import UnauthorizedException
from .not_found_exception import NotFoundException



from .custom_exception import CustomException


class UnauthorizedException(CustomException):

    status_code = 401
    message = "Not authorized to access the resourse"

    def __init__(self):
        super().__init__("Unauthorized")

    def serialize_errors(self):
        return [{"message": self.message}]



from setuptools import setup

setup(
    name='zone-common',
    version='0.0.13',
    description='Common utility package for zone c2c',
    author='Shreyas Chorge',
    author_email='shreyaschorge81@gmail.com',
    url='https://github.com/Shreyaschorge/zone_common',
    packages=["zone_common/exceptions", "zone_common/events"],
    package_dir={'': 'src'},
    install_requires=['werkzeug',
                      'nats-py',
                      'asyncio'
                      ],
)



from abc import ABC, abstractmethod, abstractproperty
from nats.js.api import AckPolicy, DeliverPolicy, ConsumerConfig
from nats.aio.client import Client as NATS
import json
import asyncio as aio


class Listener:

    @abstractproperty
    def subject(self):
        pass

    @abstractproperty
    def queueGroupName(self):
        pass

    @abstractmethod
    def onMessage(self, data, message) -> None:
        pass

    ack_wait = 5

    def __init__(self, client: NATS):
        self.client = client

    async def listen(self):
        js = self.client.jetstream()

        async def subscribe_handler(msg):
            subject = msg.subject
            print("Received a message on '{subject}'".format(
                subject=subject))

            parsedData = self.parseMessage(msg)
            aio.create_task(self.onMessage(parsedData, msg))

        config = ConsumerConfig(
            deliver_subject=self.queueGroupName,
            deliver_group=self.queueGroupName,
            ack_wait=self.ack_wait,
            ack_policy=AckPolicy.EXPLICIT,
            deliver_policy=DeliverPolicy.ALL,
            durable_name=self.queueGroupName
        )

        await js.subscribe(self.subject, queue=self.queueGroupName, manual_ack=True, config=config, cb=subscribe_handler)

    def parseMessage(self, msg):
        data = msg.data.decode()
        return json.loads(data)



from abc import ABC, abstractmethod
from werkzeug.exceptions import HTTPException
import typing as t

# TODO figureout t.NotRequired[str]
serialize_error = t.TypedDict("serialize_error", {
    'message': str,
    'field': t.Optional[str]
})


class CustomException(ABC, HTTPException):

    def __init__(self, message):
        HTTPException.__init__(self, message)

    @property
    @abstractmethod
    def status_code(self) -> None:
        pass

    @abstractmethod
    def serialize_errors(self) -> t.List[serialize_error]:
        pass



from abc import ABC, abstractproperty
import asyncio as aio
from nats.aio.client import Client as NATS
import json


class Publisher(ABC):

    @abstractproperty
    def subject(self):
        pass

    def __init__(self, client):
        self.client = client

    async def publish(self, data):
        parsedData = bytes(json.dumps(data), 'utf-8')
        await self.client.publish(self.subject, parsedData)
        print(f'Published [{self.subject}] : {json.dumps(data)}')




from .custom_exception import CustomException


class DatabaseException(CustomException):

    status_code = 500

    def __init__(self, message):
        super().__init__(message)
        self.message = message

    def serialize_errors(self):
        return [{"message": self.message}]
